@@ -8,20 +8,21 @@
    This file is part of the GNUstep project
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
-   */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
+*/
 
 #ifndef	INCLUDED_XGGENERIC_H
 #define	INCLUDED_XGGENERIC_H
@@ -49,11 +50,19 @@ typedef struct {
   Atom win_splash_atom;
   Atom win_override_atom;
   Atom win_topmenu_atom;
+  Atom win_popup_menu_atom;
+  Atom win_dropdown_menu_atom;
+  Atom win_tooltip_atom;
+  Atom win_notification_atom;
+  Atom win_combo_atom;
+  Atom win_dnd_atom;
 } XGWMWinTypes;
 
 typedef struct {
   Atom net_wm_state_atom;
   Atom net_wm_state_skip_taskbar_atom;
+  Atom net_wm_state_skip_pager_atom;
+  Atom net_wm_state_sticky_atom;
 } XGWMNetStates;
 
 /*
@@ -77,7 +86,14 @@ struct XGGeneric {
     unsigned    appOwnsMiniwindow:1;
     unsigned    doubleParentWindow:1;
   } flags;
+  // Time of last X event
   Time			lastTime;
+  // Approximate local time for last X event, used to decide 
+  // if the last X event time is still valid.
+  NSTimeInterval lastTimeStamp;
+  // last reference time on X server, used to prevent time drift between
+  // local machine and X server.
+  Time baseXServerTime;
   Time			lastClick;
   Window		lastClickWindow;
   int			lastClickX;
@@ -118,5 +134,10 @@ struct XGGeneric {
 #define WIN_LAYER_ABOVE_DOCK             10
 #define WIN_LAYER_MENU                   12
 
+/* NET WM State */
+#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
+#define _NET_WM_STATE_ADD           1    /* add/set property */
+#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */
+
 #endif
 
