@@ -9,19 +9,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #ifndef _GNUstep_H_XGOpenGL
@@ -41,32 +42,44 @@
 
 @interface XGGLContext : NSOpenGLContext
 {
-  GLXContext		glx_context;
-  GLXWindow		glx_drawable;
-  XGXSubWindow		*xsubwin;
-  XGGLPixelFormat 	*format;
+  GLXContext        glx_context;
+  GLXWindow         glx_drawable;
+  XGXSubWindow     *xSubWindow;
+  XGGLPixelFormat  *pixelFormat;
 }
+
+- (GLXContext)glxcontext;
+
 @end
 
 @interface XGGLPixelFormat : NSOpenGLPixelFormat
 {
-@public
-  union {
-    GLXFBConfig  *tab;
-    XVisualInfo  *visual;
-  } conf;
-  int		n_elem;
+  @public
+  union
+    {
+      GLXFBConfig  *fbconfig;
+      XVisualInfo  *visualinfo;
+    } configurations;
+
+  int configurationCount;
 }
+
+- (XVisualInfo *)xvinfo;
+- (GLXContext)createGLXContext: (XGGLContext *)share;
+- (GLXWindow) drawableForWindow: (Window)xwindowid;
+
 @end
 
 static inline int
 GSglxMinorVersion(Display *dpy)
 {
   int major, minor;
-  Bool result; 
-  result = glXQueryVersion (dpy, &major, &minor);
-  if (result == False)
-    return -1;
+
+  if (False == glXQueryVersion(dpy, &major, &minor))
+    {
+      return -1;
+    }
+
   return minor;
 }
 
