@@ -1,28 +1,32 @@
 /*
- * CairoFontEnumerator.m
- *
- * Copyright (C) 2003 Free Software Foundation, Inc.
- * August 31, 2003
- * Written by Banlu Kemiyatorn <object at gmail dot com>
- * Base on original code of Alex Malmberg
- * Rewrite: Fred Kiefer <fredkiefer@gmx.de>
- * Date: Jan 2006
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+   CairoFontEnumerator.m
+ 
+   Copyright (C) 2003 Free Software Foundation, Inc.
 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+   August 31, 2003
+   Written by Banlu Kemiyatorn <object at gmail dot com>
+   Base on original code of Alex Malmberg
+   Rewrite: Fred Kiefer <fredkiefer@gmx.de>
+   Date: Jan 2006
+ 
+   This file is part of GNUstep.
 
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02111 USA.
- */
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
+*/
 
 #include <Foundation/NSObject.h>
 #include <Foundation/NSArray.h>
@@ -53,12 +57,12 @@ NSMutableDictionary * __allFonts;
   face = [__allFonts objectForKey: name];
   if (!face)
     {
-      NSLog (@"Font not found %@", name);
+      NSDebugLog(@"Font not found %@", name);
     }
   return face;
 }
 
-// Make a GNUStep style font descriptor from a FcPattern
+// Make a GNUstep style font descriptor from a FcPattern
 static NSArray *faFromFc(FcPattern *pat)
 {
   int weight, slant, spacing, nsweight;
@@ -83,42 +87,42 @@ static NSArray *faFromFc(FcPattern *pat)
   switch (weight) 
     {
       case FC_WEIGHT_LIGHT:
-	[style appendString: @"Light"];
-	nsweight = 3;
-	break;
+        [style appendString: @"Light"];
+        nsweight = 3;
+        break;
       case FC_WEIGHT_MEDIUM:
-	nsweight = 6;
-	break;
+        nsweight = 6;
+        break;
       case FC_WEIGHT_DEMIBOLD:
-	[style appendString: @"Demibold"];
-	nsweight = 7;
-	break;
+        [style appendString: @"Demibold"];
+        nsweight = 7;
+        break;
       case FC_WEIGHT_BOLD:
-	[style appendString: @"Bold"];
-	nsweight = 9;
-	nstraits |= NSBoldFontMask;
-	break;
+        [style appendString: @"Bold"];
+        nsweight = 9;
+        nstraits |= NSBoldFontMask;
+        break;
       case FC_WEIGHT_BLACK:
-	[style appendString: @"Black"];
-	nsweight = 12;
-	nstraits |= NSBoldFontMask;
-	break;
+        [style appendString: @"Black"];
+        nsweight = 12;
+        nstraits |= NSBoldFontMask;
+        break;
       default:
-	nsweight = 6;
+        nsweight = 6;
     }
 
   switch (slant) 
     {
       case FC_SLANT_ROMAN:
-	break;
+        break;
       case FC_SLANT_ITALIC:
-	[style appendString: @"Italic"];
-	nstraits |= NSItalicFontMask;
-	break;
+        [style appendString: @"Italic"];
+        nstraits |= NSItalicFontMask;
+        break;
       case FC_SLANT_OBLIQUE:
-	[style appendString: @"Oblique"];
-	nstraits |= NSItalicFontMask;
-	break;
+        [style appendString: @"Oblique"];
+        nstraits |= NSItalicFontMask;
+        break;
     }
 
   if ([style length] > 0)
@@ -157,7 +161,7 @@ static NSArray *faFromFc(FcPattern *pat)
       char *family;
 
       if (FcPatternGetString(fs->fonts[i], FC_FAMILY, 0, (FcChar8 **)&family)
-	== FcResultMatch)
+          == FcResultMatch)
         {
           NSArray *fontArray;
 
@@ -172,19 +176,19 @@ static NSArray *faFromFc(FcPattern *pat)
               familyArray = [fcxft_allFontFamilies objectForKey: familyString];
               if (familyArray == nil)
                 {
-		  NSDebugLog(@"Found font family %@", familyString);
+                  NSDebugLog(@"Found font family %@", familyString);
                   familyArray = [[NSMutableArray alloc] init];
                   [fcxft_allFontFamilies setObject: familyArray
-					    forKey: familyString];
+                                         forKey: familyString];
                   RELEASE(familyArray);
                 }
               NSDebugLog(@"fc enumerator: adding font: %@", name);
               [familyArray addObject: fontArray];
               [fcxft_allFontNames addObject: name];      
               aFont = [[CairoFaceInfo alloc] initWithfamilyName: familyString
-		weight: [[fontArray objectAtIndex: 2] intValue]
-		traits: [[fontArray objectAtIndex: 3] unsignedIntValue]
-		pattern: fs->fonts[i]];
+                                             weight: [[fontArray objectAtIndex: 2] intValue]
+                                             traits: [[fontArray objectAtIndex: 3] unsignedIntValue]
+                                             pattern: fs->fonts[i]];
               [fcxft_allFonts setObject: aFont forKey: name];
               RELEASE(aFont);
             }
