@@ -7,26 +7,30 @@
    Part of this code have been written by:
    Tom MacSween <macsweent@sympatico.ca>
    Date August 2005
+
    This file is part of the GNU Objective C User Interface Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
-   This library is distributed in the hope that it will be useful, 
+
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
-   Boston, MA 02111 USA.
-   */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
+*/
 
-#include "w32_Events.h"
+#include <AppKit/NSEvent.h>
+#include <AppKit/NSWindow.h>
+#include "win32/WIN32Server.h"
+#include "win32/WIN32Geometry.h"
 
 @implementation WIN32Server (w32_General)
 
@@ -129,18 +133,5 @@
 - (void) decodeWM_COMMANDParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd
 {
 } 
-
-- (void) resetForGSWindowStyle:(HWND)hwnd w32Style:(DWORD)aStyle
-{
-  // to be completed for styles
-  LONG result;
-
-  ShowWindow(hwnd, SW_HIDE);
-  SetLastError(0);
-   result=SetWindowLong(hwnd, GWL_EXSTYLE, WS_EX_APPWINDOW);
-   result=SetWindowLong(hwnd, GWL_STYLE, (LONG)aStyle);
-  // should check error here...
-  ShowWindow(hwnd, SW_SHOWNORMAL);
-}
       
 @end 
