@@ -7,29 +7,33 @@
    Part of this code have been re-written by:
    Tom MacSween <macsweent@sympatico.ca>
    Date August 2005
+
    This file is part of the GNU Objective C User Interface Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
-   This library is distributed in the hope that it will be useful, 
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
-   Boston, MA 02111 USA.
-   */
 
-#include "w32_Events.h"
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
+*/
+
+#include <AppKit/NSEvent.h>
+#include <AppKit/NSWindow.h>
+#include "win32/WIN32Server.h"
+#include "win32/WIN32Geometry.h"
 
 @implementation WIN32Server (w32_text_focus)
 
-
 //- (LRESULT) decodeWM_SETTEXTParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd
 //{
 //printf("WM_SETTEXT\n");
@@ -46,7 +50,6 @@
 
 - (LRESULT) decodeWM_SETFOCUSParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd
 {
-  // reused from original author (added debug output)
   /* This message comes when the window already got focus, so we send a focus
      in event to the front end, but also mark the window as having current focus
      so that the front end doesn't try to focus the window again. */
@@ -67,16 +70,16 @@
   else
     {
       /* We need to do this directly and not send an event to the frontend - 
-	 that's too slow and allows the window state to get out of sync, 
-	 causing bad recursion problems */
+         that's too slow and allows the window state to get out of sync, 
+         causing bad recursion problems */
       NSWindow *window = GSWindowWithNumber((int)hwnd);
       if ([window canBecomeKeyWindow] == YES)
-	{
-	  NSDebugLLog(@"Focus", @"Making %d key", win_num);
-	  [window makeKeyWindow];
-	  [window makeMainWindow];
-	  [NSApp activateIgnoringOtherApps: YES];
-	}
+        {
+          NSDebugLLog(@"Focus", @"Making %d key", win_num);
+          [window makeKeyWindow];
+          [window makeMainWindow];
+          [NSApp activateIgnoringOtherApps: YES];
+        }
     }
     
   return 0;
